#include "parameter.h"
#include <A1-16.h>

#define SERVO_NO1		0x01
#define SERVO_NO2		0x02

static char select_mode = 0xFE;
static int temp_data;
static unsigned long last_time;
static int LED_counter = 1;

void setup(void){
	USB_Task_Setup();
	AIM_Task_Setup();
	Button_Setup();
	while(!Serial);
	Serial.println("Ready to DEMO!!");
}
void loop(void){
	static int temp_data;
	Button_Task();
	if(Serial.available() > 0){
		temp_data = Serial.parseInt();
		if(select_mode == MODE_POSITION_CONTROL){
			if((temp_data <= 1001) && (temp_data >= 23)){
				Serial.print("Input Position : "); Serial.println(temp_data);
				SetPositionI_JOG(SERVO_NO1, 0, temp_data);
				SetPositionI_JOG(SERVO_NO2, 0, temp_data);
			}
			else Serial.println("Input position is out of range");
		}
		else if(select_mode == MODE_SPEED_CONTROL){
			if((abs(temp_data) < 1023)){
				Serial.print("Input Speed : "); Serial.println(temp_data);
				A1_16_SetSpeed(SERVO_NO1, 0, temp_data);
				A1_16_SetSpeed(SERVO_NO2, 0, temp_data);
			}
			else Serial.println("Input speed is out of range");
		}
		else if(select_mode == MODE_POSITION_READ){
			temp_data = ReadPosition(SERVO_NO1);
			Serial.print("SERVO 1 Position : "); Serial.println(temp_data);
			temp_data = ReadPosition(SERVO_NO2);
			Serial.print("SERVO 2 Position : "); Serial.println(temp_data);
		}
		else if(select_mode == MODE_LED_CONTROL) ;
		else Serial.println("Please Select Mode!!");
	}
	if(select_mode == MODE_LED_CONTROL){
		if((millis() - last_time) > LED_FLASH_TIME){
			last_time = millis();
			LED_counter = LED_counter*2;
			if(LED_counter >= 16) LED_counter = 1;
			WriteDataRAM1(A1_16_Broadcast_ID, RAM_LED_Control, LED_counter);
		}
	}
}
//=========================== Function ================================
//======================== Setup Function =============================
void USB_Task_Setup(void){
  Serial.begin(115200);
}
void AIM_Task_Setup(void){
  //Configure A1-16 servo motor
	A1_16_Ini(115200);
}
void Button_Setup(void){
  //Configure onboard button pin
  pinMode(BUTTON1_PIN, INPUT);
  pinMode(BUTTON2_PIN, INPUT);
  pinMode(BUTTON3_PIN, INPUT);
  pinMode(BUTTON4_PIN, INPUT);
}
//========================= USB Function ==============================
void Clear_USB(void){
  while(Serial.read() != -1);
}
//========================== Button Function ===============================
void Button_Task(void){
  static unsigned char _button_timer = 0x00;
  static int _key = 0x00, _last_key = 0x00;
  _key = !digitalRead(BUTTON1_PIN) + ((!digitalRead(BUTTON2_PIN))<<1) + ((!digitalRead(BUTTON3_PIN))<<2) + ((!digitalRead(BUTTON4_PIN))<<3);
  if(_key != _last_key) _button_timer++;
  else _button_timer = 0;
  if(_button_timer > 20){
    _button_timer = 0;
    _last_key = _key;
    if(_key != 0){
      if(_last_key == KEY_MASK_BUTTON1){
      	select_mode = MODE_POSITION_CONTROL;
      	LED_counter = 1;
      	Serial.println("Demo Board in Position Control Mode");
      }
      else if(_last_key == KEY_MASK_BUTTON2){
      	select_mode = MODE_SPEED_CONTROL;
      	LED_counter = 1;
      	Serial.println("Demo Board in Speed Control Mode");
      }
      else if(_last_key == KEY_MASK_BUTTON3){
      	select_mode = MODE_POSITION_READ;
      	LED_counter = 1;
      	Serial.println("Demo Board in Position Read Mode");
      	Serial.println("Read Position : ");
				A1_16_TorqueOff(SERVO_NO1); A1_16_TorqueOff(SERVO_NO2);
				temp_data = ReadPosition(SERVO_NO1);
				Serial.print("SERVO 1 Position : "); Serial.println(temp_data);
				temp_data = ReadPosition(SERVO_NO2);
				Serial.print("SERVO 2 Position : "); Serial.println(temp_data);
      }
      else if(_last_key == KEY_MASK_BUTTON4){
      	select_mode = MODE_LED_CONTROL;
      	Serial.println("Demo Board in LED Flash Mode");
      	last_time = millis();
				WriteDataRAM1(A1_16_Broadcast_ID, RAM_Alarm_LED_Policy, 0x0f);
				WriteDataRAM1(A1_16_Broadcast_ID, RAM_LED_Control, MASK_LED0);
      }
      if(select_mode != MODE_LED_CONTROL){
      	LED_counter = 1;
      	WriteDataRAM1(A1_16_Broadcast_ID, RAM_LED_Control, MASK_LED0);
      	delay(10);
      	WriteDataRAM1(A1_16_Broadcast_ID, RAM_Alarm_LED_Policy, 0x00);
      }
    }
  }
}
